######################################################################################
######################################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# Stock-recruitment analysis
# Application to the A. salmon SR data from the Margaree river (Nova Scotia, Canada)
# Ricker model Gamma error structure with "Management" parameters
# Chapter 7 - Section 7.4

######################################################################################
######################################################################################

rm(list=ls())
library(boot)
library("BRugs")
library(MASS)



# ------------------------------------------------------------------------------------
# MCMC config.
# ------------------------------------------------------------------------------------

# Number iteration for the "burn in" phase
n.burnIn <- 10000
# Number iteration for inferences
n.iter <- 10000
thin <- 10
# Number of chains
n.Chains <- 3


# ------------------------------------------------------------------------------------
# Run model
# ------------------------------------------------------------------------------------

# Load/check model, load data and inits

model <- "model_SRMargaree_Ricker_Gamma_Management.txt"
data <- "data_SRMargaree.txt"
inits <- "inits_SRMargaree.txt"

modelCheck(model) 
modelData(data) 
modelCompile(numChains=n.Chains) 
modelInits(c(inits,inits,inits))

data <- source("data_SRMargaree.txt")
s <- data$value$s
r <- data$value$r
n <- data$value$n


# Burn in period

modelUpdate(numUpdates = n.burnIn, thin = 1) 


# Inferences

samplesSet(c("tau","Sopt","Ropt","hopt","alpha","beta","LogR","Copt")) 
modelUpdate(numUpdates = n.iter, thin = thin) 



# -----------------------------------------------------------
# Get results
# -----------------------------------------------------------

tau<-samplesSample("tau")
alpha<-samplesSample("alpha")
beta<-samplesSample("beta")
Copt<-samplesSample("Copt")
Ropt<-samplesSample("Ropt")
hopt<-samplesSample("hopt")
Sopt<-samplesSample("Sopt")

N <- length(Sopt)





# ------------------------------------------------------------
# Main statistics
# ------------------------------------------------------------

theta<-cbind(Copt,Ropt,alpha,1/beta,Sopt,hopt,(tau^-0.5))

store.result <- function(x, ...)
{  
y<-(c(mean(x),var(x)^0.5,quantile(x,pr=c(0.025,0.975))))
# print(floor(10*y)/10)
res<-y
}                             
apply(theta,2,store.result) -> tabres
tabres <- data.frame(round(t(tabres),2))
names(tabres) <- c("mean","sd","2.5%","97.5%")
row.names(tabres) <- c("Copt","Ropt","alpha","Smax","Sopt","hopt","sigma")

tabres



size.text <- 1.3


# ------------------------------------------------------------
# Fitted SR curve
# ------------------------------------------------------------

# windows()

res <- 6
name_figure <- "Ricker_SR_mgmt_Gamma_est.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(mfrow = c(1,1), oma = c(1,1,1,1))

delta<-tau*Sopt

ss=seq(10,6000,l=100)
rr05 <- NA*ss
rrm05 <- NA*ss
rrm95 <- NA*ss
rr50 <- NA*ss
rr95 <- NA*ss

for (t in 1:100){ 

Logrr <- log(ss[t]) + log(alpha) - beta*ss[t]
	La<- exp(2*Logrr  - log(ss[t]))
	Lb <- exp(Logrr - log(ss[t]))
rr<-rgamma(N,La/delta,Lb/delta)

rr05[t]<-quantile(rr,p=0.05)
rrm05[t]<-quantile(exp(Logrr),p=0.05)
rr50[t]<-quantile(rr,p=0.50)
rrm95[t]<-quantile(exp(Logrr),p=0.95)
rr95[t]<-quantile(rr,p=0.95)
} 


plot(ss,smooth(rr95),typ='l', lty = 3, col="black",
	ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n", 
	xlim = c(0,6000), ylim=c(0,10000))
lines(ss,smooth(rr05), col='red', lty = 3)
xx<-ss[c(1:100,100:1)]
yy<-c(rrm05,rrm95[100:1])
polygon(xx,yy,col="lightgray", border = NA)
lines(ss,smooth(rr50), lwd = 2)
points(s,r,pch=21, bg="black")
points(Sopt,Ropt,pch=".",col="black")

dens2d <- kde2d(x=Sopt, y=Ropt, n = 100)
contour(dens2d , nlevels = 4, drawlabels=F, col = "black", lw = 1, add = T) 

axis(side = 1, tick = T, labels = T, cex.axis=1, las=1) 
axis(side = 2, tick = T, labels = T, cex.axis=1)

mtext(side=1, text = "Stock", bty="n", line=3, cex=size.text, font = 1)
mtext(side=2, text = "Recruitment", bty="n", line=3, cex=size.text, font = 1)

dev.off()



# ------------------------------------------------------------
# Predictive
# ------------------------------------------------------------

# windows()

res <- 6
name_figure <- "Ricker_SR_mgmt_Gamma_IS.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

library("mnormt") 

theta<-cbind(Sopt,hopt,log(tau))
thetamean<-apply(theta,2,mean)
thetavar<-cov(theta)
nsimu<-N
thetasimu<-rmnorm(nsimu,thetamean,thetavar)

Sopt<-thetasimu[,1]
range(Sopt)

hopt<-thetasimu[,2]
range(hopt)

tau<-exp(thetasimu[,3])
range(tau)

Copt <- hopt*Sopt/(1-hopt)
Ropt <- Copt/hopt

alpha <- exp(Copt/(Sopt + Copt)) * ((Sopt + Copt)/Sopt)
beta <- Copt/(Sopt*(Sopt + Copt))

delta<-tau*Sopt

logimportance<-dmnorm(thetasimu, mean = thetamean, varcov= thetavar, log = T)-log(tau) 
logjointe<-0*(1:N)
logpred<-0*(1:N)

for (t in 1:n)
{ 
logmean <- log(s[t]) + log(alpha) - beta*s[t]
la<- exp(2*logmean  - log(s[t]))
lb <- exp(logmean - log(s[t]))
logjointe<-logjointe+dgamma(r[t],shape=la/delta,rate=lb/delta,log=T)
}

logjointe<-logjointe+dlnorm(Sopt,1000,100000^0.5,log=T)+dbeta(hopt,9,3,log=T)+dgamma(tau,1,0.25,log=T)+0
logpred<- logjointe-logimportance
K <- max(logpred)

estimlogypred <- K + log(mean(exp(logpred-K)))
estimlogypred

poids<-exp(logpred-K)
poids<-poids/sum(poids)


plot(	(1:nsimu)/nsimu, cumsum(sort(poids)),typ='l', 
		ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")
lines((1:nsimu)/nsimu,(1:nsimu)/nsimu, lty=2)

axis(side = 1, tick = T, labels = T, cex.axis=1, las=1) 
axis(side = 2, tick = T, labels = T, cex.axis=1)

mtext(side=1, text = "Importance weights", bty="n", line=3, cex=size.text, font = 1)
mtext(side=2, text = "Cumulative distribution", bty="n", line=3, cex=size.text, font = 1)

dev.off()
                                                                      
