#####################################################################################
#####################################################################################

# Parent E. and Rivot E. - An introduction to Hierarchcial Modeling of Ecological Data
# Chapman et al.
# 2012

# Stock-recruitment analysis
# Application to the A. salmon SR data from the Margaree river (Nova Scotia, Canada)
# Ricker model LogNormal error structure with "Management" parameters
# Chapter 7 - Section 7.3.2

######################################################################################
######################################################################################

rm(list=ls())
library(boot)
library("BRugs")
library(MASS)




# ------------------------------------------------------------------------------------
# MCMC config.
# ------------------------------------------------------------------------------------

# Number iteration for the "burn in" phase
n.burnIn <- 10000
# Number iteration for inferences
n.iter <- 10000
thin <- 10
# Number of chains
n.Chains <- 3


# ------------------------------------------------------------------------------------
# Run model
# ------------------------------------------------------------------------------------

# Load/check model, load data and inits

model <- "model_SRmargaree_Ricker_LogN_Management.txt"
data <- "data_SRMargaree.txt"

modelCheck(model) 
modelData(data) 
modelCompile(numChains=n.Chains) 
modelGenInits()

data <- source("data_SRMargaree.txt")
s <- data$value$s
r <- data$value$r
n <- data$value$n


# Burn in period

modelUpdate(numUpdates = n.burnIn, thin = 1) 


# Inferences

samplesSet(c("tau","Sopt","Ropt","hopt","alpha","beta","LogR","Copt")) 
modelUpdate(numUpdates = n.iter, thin = thin) 


# -----------------------------------------------------------
# Get results
# -----------------------------------------------------------

tau   <- samplesSample("tau")
alpha <- samplesSample("alpha")
beta  <- samplesSample("beta")
Copt  <- samplesSample("Copt")
Ropt  <- samplesSample("Ropt")
hopt  <- samplesSample("hopt")
Sopt  <- samplesSample("Sopt")

N <- length(Sopt)


# ------------------------------------------------------------
# Main statistics
# ------------------------------------------------------------

theta<-cbind(Copt,Ropt,alpha,1/beta,Sopt,hopt,(tau^-0.5))

store.result <- function(x, ...)
{  
y<-(c(mean(x),var(x)^0.5,quantile(x,pr=c(0.025,0.975))))
#print(floor(10*y)/10)
res<-y
}                             
tabres <- apply(theta,2,store.result)
tabres <- data.frame(round(t(tabres),2))
names(tabres) <- c("Mean","Sd","2.5%","97.5%")
row.names(tabres) <- c("Copt","Ropt","alpha","Smax","Sopt","hopt","sigma")

tabres



size.text <- 1.3

# ------------------------------------------------------------
# Marginal posterior and pairwise correlations
# ------------------------------------------------------------

# windows()

res <- 6
name_figure <- "Ricker_SR_mgmt_LogNormal_pannel.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

sigma <- tau^(-0.5)

x1 <- as.vector(Copt)
x2 <- as.vector(hopt)
x3 <- as.vector(sigma)

# y <-(c(mean(x),var(x)^0.5,quantile(x,pr=c(0.05,0.25,0.5,0.75,0.95))))
# print(floor(100*y)/100)

## put histograms on the diagonal

# panel.hist <- function(x, ...)
# {
#    usr <- par("usr"); on.exit(par(usr))
#    par(usr = c(usr[1:2], 0, 1.5) )
#    h <- hist(x,50, plot = T,freq=F)
#    breaks <- h$breaks; nB <- length(breaks)
#    y <- h$counts; y <- y/max(y)
#    rect(breaks[-nB], 0, breaks[-1], y, col="blue", ...)
# }

panel.dens <- function(x, ...)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(usr[1:2], 0, 1.5) )
    h <- density(x,adjust=1)
    h$y<-h$y/max(h$y)
    xlim=range(h$x)
    lines(h, col="black", ...)
}

## put (absolute) correlations on the upper panels,
## with size proportional to the correlations.

panel.cor <- function(x, y, digits=2, prefix="", cex.cor)
{
    usr <- par("usr"); on.exit(par(usr))
    par(usr = c(0, 1, 0, 1))
    r <- (cor(x, y))
    txt <- format(c(r, 0.123456789), digits=digits)[1]
    txt <- paste(prefix, txt, sep="")
    if(missing(cex.cor)) cex <- 0.8/strwidth(txt)
    cex=2
    # text(0.5, 0.5, txt, cex = cex * r)
	text(0.5, 0.5, txt, cex = cex)
}

par(pch='.')

pairs( cbind(x1,x2,x3),labels=c("C*","h*",expression(sigma)), 
	 lower.panel=panel.smooth,
	 diag.panel=panel.dens, 
	 upper.panel=panel.cor,
	 cex.labels = 2.0, font.labels=1.2)

dev.off()



# ------------------------------------------------------------
# Fitted SR curve
# ------------------------------------------------------------

# windows()

res <- 6
name_figure <- "Ricker_SR_mgmt_LogNormal_est.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

par(mfrow = c(1,1), oma = c(1,1,1,1))

ss=seq(10,6000,l=100)
rr05 <- NA*ss
rrm05 <- NA*ss
rrm95 <- NA*ss
rr50 <- NA*ss
rr95 <- NA*ss

for (t in 1:100){ 

Logrr <- log(ss[t]) + log(alpha) - beta*ss[t]
rr<-exp(Logrr+rnorm(N,0,tau^-0.5))
rr05[t]<-quantile(rr,p=0.05)
rrm05[t]<-quantile(exp(Logrr),p=0.05)
rr50[t]<-quantile(rr,p=0.50)
rrm95[t]<-quantile(exp(Logrr),p=0.95)
rr95[t]<-quantile(rr,p=0.95)
} 

plot(ss,smooth(rr95),typ='l', lty = 3, col="black",
	ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n", 
	xlim = c(0,6000), ylim=c(0,10000))
lines(ss,smooth(rr05), col="black", lty = 3)
xx<-ss[c(1:100,100:1)]
yy<-c(rrm05,rrm95[100:1])
polygon(xx,yy,col="lightgray", border = NA)
lines(ss,smooth(rr50), lwd = 2)
points(s,r, pch=21, bg="black")
points(Sopt,Ropt,pch=".",col="black")

dens2d <- kde2d(x=Sopt, y=Ropt, n = 100)
contour(dens2d , nlevels = 4, drawlabels=F, col = "black", lw = 1, add = T) 

axis(side = 1, tick = T, labels = T, cex.axis=1, las=1) 
axis(side = 2, tick = T, labels = T, cex.axis=1)

mtext(side=1, text = "Stock", bty="n", line=3, cex=size.text, font = 1)
mtext(side=2, text = "Recruitment", bty="n", line=3, cex=size.text, font = 1)

dev.off()



# ------------------------------------------------------------
# Predictive
# ------------------------------------------------------------

# windows()

res <- 6
name_figure <- "Ricker_SR_mgmt_LogNormal_IS.png"
png(filename = name_figure, height = 500*res, width = 500*res, res=72*res)

library("mnormt") 

theta<-cbind(Sopt,hopt,log(tau))
thetamean<-apply(theta,2,mean)
thetavar<-cov(theta)
nsimu<-N
thetasimu<-rmnorm(nsimu,thetamean,thetavar)

Sopt<-thetasimu[,1]
range(Sopt)

hopt<-thetasimu[,2]
range(hopt)

tau<-exp(thetasimu[,3])
range(tau)

Copt <- hopt*Sopt/(1-hopt)
Ropt <- Copt/hopt

alpha <- exp(Copt/(Sopt + Copt)) * ((Sopt + Copt)/Sopt)
beta <- Copt/(Sopt*(Sopt + Copt))

logimportance <- dmnorm(thetasimu, mean = thetamean, varcov= thetavar, log = T)-log(tau) 
logjointe <- 0*(1:N)
logpred <- 0*(1:N)

for (t in 1:n)
{ 
logmean <- log(s[t]) + log(alpha) - beta*s[t]
logjointe <- logjointe + dlnorm(r[t], logmean, tau^-0.5, log=T)
}

logjointe <- logjointe + dlnorm(Sopt,1000,100000^0.5,log=T) +dbeta(hopt,9,3,log=T)+ dgamma(tau,1,0.25,log=T)+0
logpred <- logjointe - logimportance
K <- max(logpred)

estimlogypred <- K + log(mean(exp(logpred-K)))
estimlogypred

poids <- exp(logpred-K)
poids <- poids/sum(poids)


plot(	(1:nsimu)/nsimu, cumsum(sort(poids)),typ='l', 
		ylab = "", xlab = "", bty="n", xaxt="n", yaxt="n")
lines((1:nsimu)/nsimu,(1:nsimu)/nsimu, lty=2)

axis(side = 1, tick = T, labels = T, cex.axis=1, las=1) 
axis(side = 2, tick = T, labels = T, cex.axis=1)

mtext(side=1, text = "Importance weights", bty="n", line=3, cex=size.text, font = 1)
mtext(side=2, text = "Cumulative distribution", bty="n", line=3, cex=size.text, font = 1)

dev.off()

                                                                   